from pysysdc.sdbus import SDBus

class Methods(object):
	
	@staticmethod
	def get_token():
		sd = SDBus(service_name="dc.cloud.auth", path="/dc/cloud/auth", if_name="dc.cloud.auth", method_name="Token", method_args="s", method_return="s", output_sig="s")
		return sd.send()

	@staticmethod
	def token_fetch():
		sd = SDBus(service_name="dc.cloud.auth", path="/dc/cloud/auth", if_name="dc.cloud.auth", method_name="Token", method_args="s", method_return="s", output_sig="s")
		return sd.send(first_arg="refetch")

	@staticmethod
	def set_static_hostname(hostname):
		return SDBus.set_hostname("SetStaticHostname", hostname)
	
	@staticmethod
	def set_pretty_hostname(hostname):
		return SDBus.set_hostname("SetPrettyHostname", hostname)

	@staticmethod
	def set_transient_hostname(hostname):
		return SDBus.set_hostname("SetHostname", hostname)

	# https://github.com/systemd/systemd/blob/main/src/hostname/hostnamectl.c#L261
	@staticmethod
	def get_hostname_by_attr(attr):
		return SDBus.get_hostname(attr)

	# https://manpages.ubuntu.com/manpages/impish/man5/org.freedesktop.login1.5.html
	@staticmethod
	def machine_reboot(wall_message=""):
		return SDBus.machine_reboot(wall_message)

	@staticmethod
	def machine_poweroff(wall_message=""):
		return SDBus.machine_poweroff(wall_message)

	@staticmethod
	def set_wall_message(wall_message=""):
		return SDBus.set_wall_message(wall_message)


from pysysdc.sdbus import SDBus
import logging

class Unit(object):
	__bus = None
	__unit_name = ""

	def __init__(self, unit_name):
		self.__bus = SDBus(service_name="org.freedesktop.systemd1",
			path="/org/freedesktop/systemd1",
			if_name="org.freedesktop.systemd1.Manager",
			method_name="StartUnit",
			method_args="ss"
		)
		
		self.__unit_name = "%s.service" % unit_name

	def __del__(self):
		pass

	def __manage(self, action):
		ret = self.__bus.send(service_name="org.freedesktop.systemd1",
			method_name=action,
			first_arg=self.__unit_name,
			second_arg="replace",
			output_sig="o"
		)

		return ret

	# https://www.freedesktop.org/wiki/Software/systemd/dbus/
	def restart(self):
		action = "%sUnit" % self.restart.__name__.title()
		return self.__manage(action)

	def stop(self):
		action = "%sUnit" % self.stop.__name__.title()
		return self.__manage(action)
	
	def start(self):
		action = "%sUnit" % self.start.__name__.title()
		return self.__manage(action)

	def reload(self):
		action = "%sUnit" % self.reload.__name__.title()
		return self.__manage(action)

	def disable(self):
		return self.__bus.unit_disable(self.__unit_name)
	
	def enable(self):
		return self.__bus.unit_enable(self.__unit_name)

	def mask(self):
		return self.__bus.unit_mask(self.__unit_name)

	def unmask(self):
		return self.__bus.unit_unmask(self.__unit_name)

	def status(self):
		return self.__bus.unit_status(self.__unit_name)


import logging
import time
import pysysdc

class SDBus(object):

	def __init__(self, service_name: str = "", path: str = "", if_name: str = "", method_name: str = "", method_args: str = "", method_return: str = "", output_sig: str = "s"):
		logging.info("pysysdc: init")
		self.service_name = service_name
		self.path = path
		self.if_name = if_name
		self.method_name = method_name
		self.method_return = method_return
		self.method_args = method_args
		self.output_sig = output_sig
		pysysdc.init()

	def __del__(self):
		pysysdc.deinit()

	def set_func(self, cf):
		pysysdc.set_func(cf)

	def __service_register(self):
		ret = -1
		while ret < 0:
			ret = pysysdc.service_register(self.path, self.if_name, self.method_name, self.method_return, self.method_args)
			if ret < 0:
				logging.error("pysysdc: '%s'", pysysdc.get_last_error(ret))
				pysysdc.deinit()
				time.sleep(3)

		return True

	def listen(self):
		self.__service_register()
		while True:
			ret = pysysdc.listen()
			if ret < 0:
				err = pysysdc.get_last_error(ret)
				logging.error("pysysdc: '%s'", err)
				return False, err

		return True, ''

	def send(self, service_name: str = "", 
					path: str = "", 
					if_name: str = "", 
					method_name: str = "", 
					method_args: str = "", 
					output_sig: str = "s",
					first_arg = "", 
					second_arg = ""):
		ret = pysysdc.send(
			service_name if service_name else self.service_name,
			path if path else self.path, 
			if_name if if_name else self.if_name, 
			method_name if method_name else self.method_name, 
			method_args if method_args else self.method_args,
			output_sig if output_sig else self.output_sig,
			first_arg,
			second_arg)

		if ret[0] < 0:
			err = pysysdc.get_last_error(ret)
			logging.error("pysysdc: '%s'", err)

		logging.debug("pysysdc: send '%s'", ret)
		return ret

	def unit_enable(self, unit_name):
		return pysysdc.unit_enable(unit_name)

	def unit_disable(self, unit_name):
		return pysysdc.unit_disable(unit_name)

	def unit_mask(self, unit_name):
		return pysysdc.unit_mask(unit_name)

	def unit_unmask(self, unit_name):
		return pysysdc.unit_unmask(unit_name)

	def unit_status(self, unit_name):
		return pysysdc.unit_status(unit_name)

	@staticmethod
	def set_hostname(method, value):
		if not value:
			return None
		return pysysdc.set_hostname(method, value)

	@staticmethod
	def get_hostname(attr):
		return pysysdc.get_hostname(attr)

	@staticmethod
	def machine_reboot(wall_message=""):
		return pysysdc.machine_reboot(wall_message)

	@staticmethod
	def machine_poweroff(wall_message=""):
		return pysysdc.machine_poweroff(wall_message)

	@staticmethod
	def set_wall_message(wall_message=""):
		return pysysdc.set_wall_message(wall_message)


#!/usr/bin/env python3

import os
from setuptools import setup, Extension
from pathlib import Path
import re

VERSION = '1.0.0'
with open('pysysdc/__version__.py', 'r') as f:
	VERSION = f.read()

VERSION = re.sub(r'[^0-9.]+', '', VERSION)

long_description = (Path(os.getcwd()) / "README.md").read_text()

setup(
	name="pysysdc",
	version=VERSION,
	description="Python bindings for sd-bus",
	long_description=long_description,
	include_package_data=True,
	author_email="andrey@bagrintsev.me",
	long_description_content_type='text/markdown',
	author=[
		"Andrey Bagrintsev"
	],
	url="https://github.com/sb0y/pysysdc",
	license="BSD",
	packages=["pysysdc"],
	classifiers=[
		"Intended Audience :: Developers",
		"Topic :: Software Development :: Libraries :: Python Modules",
		"License :: OSI Approved :: GNU General Public License (GPL)",
		"Development Status :: 5 - Production/Stable",
		"Operating System :: Unix",
		"Programming Language :: C",
	],
	ext_modules=[
		Extension(
			"_pysysdc",
			[
				"pysysdc/pysysdc.c",
				"pysysdc/sdbus.c",
				"pysysdc/python_runner.c",
				"pysysdc/systemd_methods.c",
				"pysysdc/dc_error_handling.c",
				"pysysdc/helpers.c",
				"pysysdc/connect.c"
			],
			libraries=['systemd'],
			#extra_compile_args=["-O3", "-std=c99"] # match systemd
		)
	],
	scripts=['scripts/dcbus'],
	data_files=[("/etc/dbus-1/",
		["etc/dbus-1/system-local.conf"])]
)
